#ifndef ZASM_PROCESSOR_H
#define ZASM_PROCESSOR_H 1

#include <zasm/list.h>

enum
  {
    PROC_LITTLE_ENDIAN,
    PROC_BIG_ENDIAN
  };

#define PROCESSOR_T(O) ((struct processor_t *)(O))
struct processor_t
{
  // public
  int endianness;
  struct list_t *instruction_set;
};

#define PROC_INSTR_T(O) ((struct proc_instr_t *)(O))
struct proc_instr_t
{
  // public
  char *name;
  struct list_t *arguments;
};

#define PROC_INSTR_ARG_T(O) ((struct proc_instr_arg_t *)(O))
struct proc_instr_arg_t
{
  // public
  struct list_t *pattern;
  struct list_t *data;
};

#include <zasm/feeder.h>

struct processor_t *processor_new();
void processor_fill(struct processor_t *proc);
struct processor_t *processor_read(char *filename);
void processor_add_instructions(struct processor_t *proc, struct list_t *instrs);
struct token_t processor_instructions_p(struct feeder_t *input, struct token_t *token);
struct token_t processor_p(struct feeder_t *input, struct token_t *token);
struct proc_instr_t *processor_get_instr(struct processor_t *proc, char *instr);
void processor_delete(struct processor_t *proc);
void processor_free(struct processor_t *proc);

struct proc_instr_t *proc_instr_new();
void proc_instr_fill(struct proc_instr_t *instr);
struct proc_instr_arg_t *proc_instr_get_arg(struct proc_instr_t *instr, struct list_t *args);
void proc_instr_delete(struct proc_instr_t *instr);
void proc_instr_free(struct proc_instr_t *instr);

struct proc_instr_arg_t *proc_instr_arg_new();
void proc_instr_arg_fill(struct proc_instr_arg_t *instr);
int proc_instr_arg_check_pattern(struct list_t *pat, struct list_t *args);
struct list_t *proc_instr_arg_get_pattern(struct list_t *pat, struct list_t *args);
void proc_instr_arg_delete(struct proc_instr_arg_t *instr);
void proc_instr_arg_free(struct proc_instr_arg_t *instr);

#endif
