#define ZASM_PP_NAMESPACE_C 1

#include <zasm/pp_namespace.h>

struct pp_namespace_t *pp_namespace_new()
{
  struct pp_namespace_t *val;

  ALLOCATE(struct pp_namespace_t, val, 1);
  pp_namespace_fill(val);
  return val;
}

void pp_namespace_fill(struct pp_namespace_t *ns)
{
  namespace_fill(&(ns->namespace));
}

void pp_namespace_add_level(struct pp_namespace_t *ns)
{
  namespace_add_level(&(ns->namespace), NS_LEVEL_T(pp_level_new(ns)));
}

void pp_namespace_del_level(struct pp_namespace_t *ns)
{
  pp_level_free(PP_LEVEL_T(namespace_del_level(&(ns->namespace))));
}

void pp_namespace_define(struct pp_namespace_t *ns, char *name, struct list_t *args, struct list_t *body)
{
  struct pp_entry_t **entry;

  entry = PP_ENTRY_PT(hash_table_lookup(&(ns->namespace.local->hash_table), name));
  if (*entry)
    {
      WARNING("duplicate macro \"%s\", using first defined value\n", name);
      return;
    }
  *entry = pp_entry_new(PP_LEVEL_T(ns->namespace.local), name, pp_macro_new(PP_LEVEL_T(ns->namespace.local), args, body));
}

int  pp_namespace_defined(struct pp_namespace_t *ns, char *name)
{
  struct pp_entry_t **entry;

  entry = PP_ENTRY_PT(namespace_lookup(&(ns->namespace), name, NULL));
  if (*entry)
    return 1;
  return 0;
}

void pp_namespace_redef(struct pp_namespace_t *ns, char *name, struct list_t *args, struct list_t *body)
{
  struct pp_entry_t **entry;
  struct ns_level_t *level;

  entry = PP_ENTRY_PT(namespace_lookup(&(ns->namespace), name, &level));
  if (!(*entry))
    {
      WARNING("redefined macro \"%s\" did not exist\n", name);
      *entry = pp_entry_new(PP_LEVEL_T(level), name, pp_macro_new(PP_LEVEL_T(level), args, body));
    }
  else
    pp_entry_redef(*entry, pp_macro_new(PP_LEVEL_T(level), args, body));
}

void pp_namespace_del(struct pp_namespace_t *ns, char *name)
{
  struct pp_entry_t **entry;

  entry = PP_ENTRY_PT(namespace_lookup(&(ns->namespace), name, NULL));
  if (!(*entry))
    WARNING("deleted macro \"%s\" did not exist\n", name);
  else
    {
      hash_entry_unlink((struct hash_entry_t **)entry);
      pp_entry_free(*entry);
    }
}

struct pp_macro_t *pp_namespace_lookup(struct pp_namespace_t *ns, char *name)
{
  struct pp_entry_t **entry;

  entry = PP_ENTRY_PT(namespace_lookup(&(ns->namespace), name, NULL));
  if (*entry)
    return (*entry)->macro;
  else
    return NULL;
}

struct pp_macro_t *pp_namespace_lookup_local(struct pp_namespace_t *ns, char *name)
{
  struct pp_entry_t **entry;

  entry = PP_ENTRY_PT(hash_table_lookup(&(ns->namespace.local->hash_table), name));
  if (*entry)
    return (*entry)->macro;
  else
    return NULL;
}

void pp_namespace_delete(struct pp_namespace_t *ns)
{
  namespace_delete(&(ns->namespace));
}

void pp_namespace_free(struct pp_namespace_t *ns)
{
  pp_namespace_delete(ns);
  free(ns);
}

struct pp_level_t *pp_level_new(struct pp_namespace_t *parent)
{
  struct pp_level_t *val;

  ALLOCATE(struct pp_level_t, val, 1);
  pp_level_fill(val, parent);
  return val;
}

void pp_level_fill(struct pp_level_t *level, struct pp_namespace_t *parent)
{
  ns_level_fill(&(level->level), &(parent->namespace));
  level->string_alloc = NULL;
  level->entry_alloc = NULL;
  level->macro_alloc = NULL;
}

void pp_level_delete(struct pp_level_t *level)
{
  ns_level_delete(&(level->level));
  string_alloc_free(level->string_alloc);
  pp_entry_alloc_free(level->entry_alloc);
  pp_macro_alloc_free(level->macro_alloc);
}

void pp_level_free(struct pp_level_t *level)
{
  pp_level_delete(level);
  free(level);
}
