#define ZASM_STREAM_C 1

#include <zasm/stream.h>
#include <zasm/memory.h>

struct action_t *action_new(action_execute_t execute,
			    struct feeder_t *input, struct stream_t *output, void *data)
{
  struct action_t *val;

  ALLOCATE(struct action_t, val, 1);
  action_fill(val, execute, input, output, data);
  return val;
}

void action_fill(struct action_t *val,
		 action_execute_t execute,
		 struct feeder_t *input, struct stream_t *output, void *data)
{
  val->data = data;
  val->execute = execute;
  val->input = input;
  val->output = output;
  return;
}

void action_connect(struct action_t *action, struct feeder_t *input, struct stream_t *output)
{
  action->input = input;
  action->output = output;
}

void action_execute(struct action_t *action)
{
  action->execute(action->input, action->output, action->data);
}

void action_delete(struct action_t *action)
{}

void action_free(struct action_t *action)
{
  free(action);
}


struct stream_t *stream_new   (struct action_t *cb_read, struct action_t *cb_wrote)
{
  struct stream_t *val;

  ALLOCATE(struct stream_t, val, 1);
  stream_fill(val, cb_read, cb_wrote);
  return val;
}

void stream_fill(struct stream_t *val, struct action_t *cb_read, struct action_t *cb_wrote)
{
  list_fill(&(val->list));
  val->cb_read = cb_read;
  val->cb_wrote = cb_wrote;
}

struct token_t stream_get(struct stream_t *stream)
{
  struct token_t tok;

  tok = list_get_begin(&(stream->list));
  while ((tok.type == TOK_LIST_END) && (stream->cb_read))
    {
      action_execute(stream->cb_read);
      tok = list_get_begin(&(stream->list));
    }
  if (tok.type == TOK_LIST_END)
    tok.type = TOK_STREAM_END;
  return tok;
}

void stream_unget(struct stream_t *stream, struct token_t token)
{
  list_add_begin(&(stream->list), token);
}

void stream_put(struct stream_t *stream, struct token_t token)
{
  list_add_end(&(stream->list), token);
  if (stream->cb_wrote)
    action_execute(stream->cb_wrote);
}

void stream_delete(struct stream_t *stream)
{
  list_delete(&(stream->list));
}

void stream_free(struct stream_t *stream)
{
  stream_delete(stream);
  free(stream);
}
